@@ -7,12 +7,13 @@ from muffin.plugins import BasePlugin
 from peewee import Model
 from peewee_migrate import Router
 from peewee_aio.model import AIOModel
-from peewee_aio.manager import Manager, cached_property
+from peewee_aio.manager import Manager, TMODEL
+from aio_databases.database import Database, ConnectionContext, TransactionContext
 
 from .fields import JSONField, Choices
 
 
-__version__ = "0.4.0"
+__version__ = "0.5.0"
 __project__ = "muffin-peewee-aio"
 __author__ = "Kirill Klenov <horneds@gmail.com>"
 __license__ = "MIT"
@@ -46,6 +47,7 @@ class Plugin(BasePlugin):
 
     router: t.Optional[Router] = None
     manager: Manager = Manager('dummy://localhost')  # Dummy manager for support registration
+    database: Database
 
     def setup(self, app: muffin.Application, **options):
         """Init the plugin."""
@@ -56,13 +58,14 @@ class Plugin(BasePlugin):
         for model in list(self.manager):
             manager.register(model)
         self.manager = manager
+        self.database = manager.aio_database
+        self.Model: t.Type[AIOModel] = self.manager.Model
 
         if self.cfg.migrations_enabled:
             router = Router(manager.pw_database, migrate_dir=self.cfg.migrations_path)
             self.router = router
 
             # Register migration commands
-            @app.manage
             def pw_migrate(name: str = None, fake: bool = False):
                 """Run application's migrations.
 
@@ -72,7 +75,8 @@ class Plugin(BasePlugin):
                 with manager.allow_sync():
                     router.run(name, fake=fake)
 
-            @app.manage
+            app.manage(pw_migrate)
+
             def pw_create(name: str = 'auto', auto: bool = False):
                 """Create a migration.
 
@@ -80,9 +84,10 @@ class Plugin(BasePlugin):
                 :param auto: Track changes and setup migrations automatically
                 """
                 with manager.allow_sync():
-                    router.create(name, auto and [m for m in self.models])
+                    router.create(name, auto and [m for m in self.manager.models])
+
+            app.manage(pw_create)
 
-            @app.manage
             def pw_rollback(name: str = None):
                 """Rollback a migration.
 
@@ -91,65 +96,69 @@ class Plugin(BasePlugin):
                 with manager.allow_sync():
                     router.rollback(name)
 
-            @app.manage
+            app.manage(pw_rollback)
+
             def pw_list():
                 """List migrations."""
                 with manager.allow_sync():
                     router.logger.info('Migrations are done:')
-                    router.logger.info('\n'.join(self.router.done))
+                    router.logger.info('\n'.join(router.done))
                     router.logger.info('')
                     router.logger.info('Migrations are undone:')
-                    router.logger.info('\n'.join(self.router.diff))
+                    router.logger.info('\n'.join(router.diff))
+
+            app.manage(pw_list)
 
         if self.cfg.auto_connection:
             app.middleware(self.get_middleware(), insert_first=True)
 
     async def startup(self):
         """Connect to the database (initialize a pool and etc)."""
-        await self.manager.connect()
+        await self.database.connect()
 
     async def shutdown(self):
         """Disconnect from the database (close a pool and etc.)."""
-        await self.manager.disconnect()
+        await self.database.disconnect()
 
     async def __aenter__(self) -> 'Plugin':
         """Connect the database."""
-        await self.manager.aio_database.connect()
+        await self.database.connect()
         return self
 
-    async def __aexit__(self, *args):
+    async def __aexit__(self, *_):
         """Disconnect the database."""
-        await self.manager.aio_database.disconnect()
+        await self.database.disconnect()
 
-    def __getattr__(self, name: str) -> t.Any:
-        """Proxy attrs to self database."""
-        return getattr(self.manager, name)
+    def register(self, Model: t.Type[TMODEL]) -> t.Type[TMODEL]:
+        """Register a model with self manager."""
+        return self.manager.register(Model)
 
-    async def create_tables(self, *Models: Model):
+    def connection(self, *params, **opts) -> ConnectionContext:
+        return self.database.connection(*params, **opts)
+
+    def transaction(self, *params, **opts) -> TransactionContext:
+        return self.database.transaction(*params, **opts)
+
+    async def create_tables(self, *Models: t.Type[Model]):
         """Create SQL tables."""
-        await self.manager.create_tables(*(Models or list(self.manager)))
+        await self.manager.create_tables(*(Models or self.manager.models))
 
-    async def drop_tables(self, *Models: Model):
+    async def drop_tables(self, *Models: t.Type[Model]):
         """Drop SQL tables."""
-        await self.manager.drop_tables(*(Models or list(self.manager)))
+        await self.manager.drop_tables(*(Models or self.manager.models))
 
     def get_middleware(self) -> t.Callable:
         """Generate a middleware to manage connection/transaction."""
 
-        async def middleware(handler, request, receive, send):
-            async with self.manager.connection():
+        async def middleware(handler, request, receive, send):  # type: ignore
+            async with self.connection():
                 return await handler(request, receive, send)
 
         if self.cfg.auto_transaction:
 
             async def middleware(handler, request, receive, send):  # noqa
-                async with self.manager.connection():
-                    async with self.manager.transaction():
+                async with self.connection():
+                    async with self.transaction():
                         return await handler(request, receive, send)
 
         return middleware
-
-    @cached_property
-    def Model(self) -> AIOModel:
-        """Generate base async model class."""
-        return self.manager.Model
